format binary as "img"
	use16
        org     0x7C00                   ;bios puts us here! 
start: 
	;mov     ax, 0x1112             ; Set 80x50 mode 
	;int   0x10                     ; Do it!

        push    cs 
        pop     ds                      ; ds=cs (0) 

        mov    byte[boot_drive], dl     ; save the boot drive number 

        mov si, loading_msg             ; Print a message 
        call Print 
.load: 
        mov     bx, 0x7E00              ; offset= 7E00 

        mov     dl, byte[boot_drive] 
        mov     dh, 0                   ; set Head 
        mov     cl, 2                   ; set Sector 
        mov     ch, 0                   ; set track 

        mov     ah, 2                   ; bios int 0x13 function ah=2, read sectors
        mov     al, 13                  ; read 13 sector 

        int     0x13                    ; call bios 
        jnc     ok_load_setup           ; if ok - continue 

        mov 	si, error_msg           ; Print a message 
        call 	Print
ok_load_setup: 
        mov 	si, ok_msg
        call 	Print 
	mov	si, paging_msg
	call	Print
	cli
;--------------------------------------;
; ********** Set up paging *********** ;
;--------------------------------------;
; I'll map 4MB of memory...
; use 4kb pages...
	xor	 dx, dx
	mov	 di, 0x1000	       ; Paging starts at 0x1000 (at the PML4)
	mov	cr3, edi	       ; place the location of PML4 into Cr3
	push	edi		       ; [!] put 0x00001000 on the stack. [!]
	;'''''''''''''''''''''''''''''';
	; Create the page directories. ;
	;,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,;
	xor	eax, eax
	mov	 al, 0x2	       ; NOT present
	mov	 cx, 1024	       ; 1024 entries... 4MB
	push	 cx		       ; {!} put 1024 onto the stack. {!}
	rep	stosd		       ; Clear all 1024 entries
	;'''''''''''''''''''''''''''''';
	; Create 1 page table, for 4MB ;
	;,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,;
	pop	 cx		       ; {!} Get 1024 from the stack. {!}
	mov	 di, 0x2000
.loop:
	or	eax, 3
	stosd
	add	eax, 4096
	loop	.loop
	;'''''''''''''''''''''''''''''';
	; Point page DIR to page table ;
	;,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,;
	pop	edi		       ; [!] Get 0x00001000 from the stack [!]
	mov	eax, 0x2003
	stosd
; page_directory   = 0x1000
; first_page_table = 0x2000
        mov 	si, ok_msg
        call 	Print 
	mov	si, Pmode_msg
	call	Print
;--------------------------------------;
; ********* Install The IDT ********** ;
;--------------------------------------;
	xor	edi, edi	       ; IDT is placed at 0x00000000
	mov	esi, IDT_snip	       ; The IDT snip
	mov	 cx, 256	       ; 2048 bytes / 4 (4 byte regs)
	lodsd			       ; get the first half of the IDT snip
	xchg	eax, ebx
	lodsd			       ; Now the second part.
	xchg	ebx, eax	       ; Have EAX hold the first part again.
.idtloop:
	stosd			       ; Store one part of the IDT
	xchg	eax, ebx	       ; Swap to the other part.
	loop	.idtloop	       ; Do this until CX == 0
;--------------------------------------;
; ********** Program the PIT ********* ;
; Set PIT to 100Hz (0x2E 94)           ;
;--------------------------------------;
	;'''''''''''''''''''''''''''''';
	; Set low byte.                ;
	;,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,;
	mov	 al, 00010110b	       ; control register
	out    0x43, al 
	mov	 al, 0x2E	       ; LSB of clock divisor
	out    0x40, al
	;'''''''''''''''''''''''''''''';
	; Set high byte.               ;
	;,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,;
	mov	 al, 00100110b	       ; Contol register.
	out    0x43, al
	mov	 al, 0x9A	       ; MSB only clock divisor
	out    0x40, al 	       ; Wait for PIT to finish...
;--------------------------------------;
; ************ Map the PIC *********** ;
;--------------------------------------;
	mov	 al, 00010001b	       ; bit 1: Use ICW4
				       ; bit 5: Intalise PIC
	out    0x20, al
	mov	 al, 32 	       ; Tell PIC 1 what ISR number IRQ 0 is at
	out    0x21, al
	mov	 al, 40
	out    0xA1, al 	       ; Tell PIC 2 what ISR number IRQ 8 is at

	mov	 al, 0x4	       ; Tell the first PIC what line to
				       ; casscade
	out    0x21, al
	mov	 al, 0x2	       ; Tell the second PIC what line to
				       ; cascade
	out    0xA1, al
	mov	 al, 1		       ; Operate in intel (x86)
	out    0x21, al
	out    0xA1, al
	mov	 al, 11111111b	       ; Disable all IRQ's 
	out    0x21, al
	out    0xA1, al
;--------------------------------------;
; ******** Open the A-20 gate ******** ;
;--------------------------------------;
	mov	 al, 0xAD	       ; Disable the system keyboard.
	out    0x64, al
	call	wait_for_8042	       ; Wait until the beyboard is ready.
	mov	 al, 0xD1	       ; Instruct the keyboard to write to pin2
	out    0x64, al

	call	wait_for_8042	       ; Wait for the system keyboard.
	mov	 al, 0xDF	       ; Enable the A-20 address line.
	out    0x60, al

	mov	 al, 0xAE	       ; Re-enable the keyboard.
	out    0x64, al
	call	wait_for_8042	       ; Wait for it.
;--------------------------------------;
; Pass boot drive to kernel...         ;
;--------------------------------------;
	xor	ax, ax
	mov	al, [boot_drive]
	push	ax
;--------------------------------------;
; ********** Enter P-mode ************ ;
;--------------------------------------;
	LGDT	[gdtr]		       ; Load the GDT
	mov	eax, cr0
	or	eax, 0x80000001        ; Enable PE bit, and paging bit.
	mov	cr0, eax
;--------------------------------------;
; Jump to kernel.                      ;
;--------------------------------------;
	jmp	0x0008:0x7E00
;=============================================================================;
; Routeens.                                                                   ;
;=============================================================================;
	use16
Print: 
        mov	ah,0x0E 
        mov	bx,0x0007 
.loop:
        lodsb 
        cmp 	al,0
        je  	Print_end 
        int 	0x10 
        jmp 	.loop
Print_end: 
        ret
wait_for_8042:
	jmp	$+2		       ; wait for second

	in	 al, 0x64	       ; read from KBD
	test	 al, 2		       ; is it 2?
	jnz	wait_for_8042	       ; If not wait...
	ret
	use32
;=============================================================================;
; Data.                                                                       ;
;=============================================================================;
 
loading_msg             db 'Loading sectors...',0
paging_msg		db 'Installing paging...  ',0
Pmode_msg		db 'Entering P-mode...',0
ok_msg                  db '              OK!',13,10,0
error_msg               db 13,10,'Boot Error!',0 

boot_drive              db 0
;-----------------------------------------------------------------------------;
; *********************************** GDT *********************************** ;
;-----------------------------------------------------------------------------;
gdtr:	dw GDT_end - GDT_start - 1     ; GDT size - 1  So that should be 23.
	dd GDT_start		       ; Were the GDT starts.
GDT_start:
	dq 0x0000000000000000	       ; Null descriptor.
	dw 0x03FF		       ; 4MB limmit  1023 (1024-1)*4096 = 4MB)
	dw 0x0000		       ; Base adress = 0x0000
	dw 0x9A00		       ; code read/exec
	dw 0x00C0		       ; granularity

	dw 0x03FF		       ; 4MB limmit  1023 ((1024-1)*4096=2MB)
	dw 0x0000		       ; Base adress = 0x0000
	dw 0x9200		       ; code read/exec
	dw 0x00C0		       ; granularity
GDT_end:
;-----------------------------------------------------------------------------;
; *********************************** IDT *********************************** ;
;-----------------------------------------------------------------------------;
	use32
idtr:	dw 2048 		       ; Size of IDT
	dd 0x00000000		       ; Location of IDT.
IDT_snip:
	dw dummy_int		       ; Interupt location.
	dw 0x0008		       ; 0x0008 = offset of the code descriptor
				       ; in the GDT.
	dw 0x8E00		       ; This is required.
	dw 0			       ; This needs to be a zero.
dummy_int:
	mov	 ax, 0x0058	       ; AH == 0, print Char
                                       ; AL == X, letter to print.
	int	0x70
	cli
	hlt


times (510-($-$$)) db 0
       dw 0xAA55       

Kernel:
file 'kernel.bin'

	times (368640-($-$$)) db 0     ; Create a 360kb floppy.
